@@ -5,11 +5,13 @@ from . import ui
 from . import analyze
 from . import read
 
-from ._version import version as __version__
+from ._version import __version__
 
 from . import control
 from . import scan
 from . import pipeline
 from . import tasks
 
-__all__ = ["analyze", "plot", "ui", "scan", "control", "pipeline", "tasks"]
+from . import cli
+
+__all__ = ["analyze", "plot", "ui", "scan", "control", "pipeline", "tasks", "cli"]
