@@ -9,9 +9,10 @@ import matplotlib
 from matplotlib import pyplot as plt
 from matplotlib.colors import LogNorm
 from matplotlib.backends.backend_pdf import PdfPages
+from rich import print
 
 from .analyze import calculate_CFD_using_FF
-from .read import Event, Settings
+from .read import Event, Settings, Stats
 
 
 DATA_POINTS_PER_FPGA_CYCLE = 5
@@ -33,7 +34,7 @@ def filter_events(
         return events[:N]
 
     print(
-        f"[Warning] plotting: got less events than requested for plotting {N_events} < {N}"
+        f"[orange3]WARNING[/] plotting: got less events than requested for plotting {N_events} < {N}"
     )
     return events
 
@@ -108,7 +109,7 @@ def create_persistent_plot(
     """
     N = len(traces)
     if not N:
-        print("[Warning] no traces in persistince plot", flush=True)
+        print("[orange3]WARNING[/] no traces in persistince plot", flush=True)
     mytime = np.linspace(0, 2 * (traces.shape[1] - 1), traces.shape[1])  # in ns
 
     if x_range is None:
@@ -195,16 +196,16 @@ def find_sums(
             s1 = np.sum(t[i : i + Ls])
             s2 = np.sum(t[i + Ls : i + Ls + Gs])
             s3 = np.sum(t[i + Ls + Gs : i + Ls + Gs + Ls])
-            # print([s1, s2, s3], [Esum1, Esum2, Esum3])
+            # t([s1, s2, s3], [Tsum, Gsum, Lsum], flush=True)
             if [s1, s2, s3] == [Tsum, Gsum, Lsum]:
                 sumIDX.append(i)
                 Esums.append([s1, s2, s3])
                 # print(f'found it at {sumIDX}')
                 break
         else:
-            print("[ERROR] Could not find sums!")
+            print("[red]ERROR[/] Could not find sums!")
             print(
-                "[ERROR] Try increasing trace length and/or trace delay and run again"
+                "[red]ERROR[/] Try increasing trace length and/or trace delay and run again"
             )
             print(
                 "Ideal settings for offline computation: trace length > 2*Ls+Gs"
@@ -253,7 +254,7 @@ def MCA(
         fig, ax = plt.subplots()
     data = data.reshape(-1, rebin).sum(axis=1)
     ax.plot(data, label=label)
-    if log:
+    if log and np.any(data > 0):
         ax.set_yscale("log")
     ax.set_xlabel("Channel")
     ax.set_ylabel("Counts")
@@ -309,15 +310,17 @@ def CFD(
     channels = {e.channel for e in events}
     if len(channels) != 1:
         print(
-            "[ERROR] Can only plot CFD for events from a single channel at the moment"
+            "[red]ERROR[/] Can only plot CFD for events from a single channel at the moment"
         )
         return
     channel = list(channels)[0]
 
-    CFDth = setting.CFDThresh[channel]
-    Lf = int(setting.FastLength[channel])
-    Gf = int(setting.FastGap[channel])
-    FFth = setting.FastThresh[channel]  # TODO: check scaling, perhaps x/L*5
+    CFDth = setting.get_by_name("CFDThresh")[channel]
+    Lf = int(setting.get_by_name("FastLength")[channel])
+    Gf = int(setting.get_by_name("FastGap")[channel])
+    FFth = setting.get_by_name("FastThresh")[
+        channel
+    ]  # TODO: check scaling, perhaps x/L*5
 
     # parameters needed for CFD calculation
     w = [w]
@@ -328,7 +331,7 @@ def CFD(
     traces = [e.trace for e in events]
     for t in traces:
         if t is None:
-            print("[ERROR] Missing trace...skip plotting")
+            print("[red]ERROR[/] Missing trace...skip plotting")
             return
     traces = np.asarray(traces)
 
@@ -378,6 +381,7 @@ def CFD(
 def energy_sums(
     events: Iterable[Event],
     setting: Settings,
+    stats: Stats,
     N: Optional[int] = None,
     randomize: bool = False,
     persistent: bool = True,
@@ -425,15 +429,25 @@ def energy_sums(
     # align data
     Ls, Gs, ch, trace, trailsum, gapsum, leadsum = [], [], [], [], [], [], []
 
-    slow_filter_range = setting.SlowFilterRange
+    slow_filter_range = setting.get_by_name("SlowFilterRange")
     for e in events:
         if e.CFD_error == 0:
             # convert Ls and Gs into number of data points
             Ls.append(
-                int(setting.SlowLength[e.channel] * (2 ** slow_filter_range) * 10 / 2)
+                int(
+                    setting.get_by_name("SlowLength")[e.channel]
+                    * (2 ** slow_filter_range)
+                    * 10
+                    / 2
+                )
             )
             Gs.append(
-                int(setting.SlowGap[e.channel] * (2 ** slow_filter_range) * 10 / 2)
+                int(
+                    setting.get_by_name("SlowGap")[e.channel]
+                    * (2 ** slow_filter_range)
+                    * 10
+                    / 2
+                )
             )
             ch.append(e.channel)
             trace.append(e.trace)
@@ -448,7 +462,7 @@ def energy_sums(
         return
     IDX = np.arange(0, trace.shape[0], 1)
     if len(trailsum) == 0:
-        print("[ERROR] Raw energy sums must be anabled in the PIXIE-16")
+        print("[red]ERROR[/] Raw energy sums must be anabled in the PIXIE-16")
         return
 
     try:
@@ -492,13 +506,14 @@ def energy_sums(
             label="Raw energy sums",
         )
         ch = list(channels)[0]
+        live_time = stats["live_time"][ch]
         ax.set_title(
             f"channel: {ch}; number of traces: {len(trace)};"
-            f" live time: {setting.get('LiveTime', channel=ch, as_pint=True)}"
+            f" live time: {live_time:.2f} s"
         )
     else:
         print(
-            "[Warning] plot.energy_sums: more than one channel given, not drawing positions of sums"
+            "[orange3]WARNING[/] plot.energy_sums: more than one channel given, not drawing positions of sums"
         )
     ax.legend()
     ax.set_ylim(bottom=0)
