@@ -1,19 +1,21 @@
 """implementation of queues for parsing list mode data from the pixie16"""
 
 
-from collections import namedtuple
+from collections import namedtuple, deque, defaultdict
 from itertools import islice
+from pathlib import Path
 from typing import Union
 
-import pandas as pd
-
 import cbitstruct as bitstruct
 import numpy as np
-
+import pandas as pd
+from rich import print
 
 WORD_SIZE = 4
 
-
+# chunk_timestamp is a field that doesn't come from the pixie
+# it can be used to for example store a unix time stamp when the data was converted
+# tasks/SortEvents uses it for example this way.
 Event = namedtuple(
     "Event",
     [
@@ -40,8 +42,10 @@ Event = namedtuple(
         "QDCSum6",
         "QDCSum7",
         "ext_timestamp",
+        "chunk_timestamp",  # not a pixie16 field
     ],
-    defaults=(-1, -1, -1, -1.0, -1, -1, -1, -1, -1, -1, -1, -1, -1.0),
+    # note that defaults are aligned to the end and value without defaults are mandatory
+    defaults=(-1, -1, -1, -1.0, -1, -1, -1, -1, -1, -1, -1, -1, -1.0, -1.0),
 )
 
 
@@ -72,6 +76,7 @@ class WordsIO:
     def __init__(self, binary_data: Union[bytes, bytearray] = b""):
         """Initialize queue"""
         self.binary_data = bytearray(binary_data)
+        self.last_data = deque(maxlen=5)  # used for debugging
 
     def __len__(self):
         """Number of bytes in queue"""
@@ -85,13 +90,21 @@ class WordsIO:
         """Put more bytes into queue"""
         self.binary_data.extend(additional_binary_data)
 
-    def pop(self, size, raw=False):
+    def pop(self, size: int, raw=False):
         """Pop the top size words from queue"""
         res = self.peek(size, raw=raw)
         del self.binary_data[: size * WORD_SIZE]
+        self.last_data.append(res)
         return res
 
-    def peek(self, size, raw=False):
+    def assert_data_is_available(self, size: int):
+        """Make sure `size` words are available."""
+        if size * WORD_SIZE > len(self):
+            raise LeftoverBytesError(
+                f"Requested {size} words, only {len(self)//WORD_SIZE} words in object"
+            )
+
+    def peek(self, size: int, raw=False):
         """Get the top size words from queue without removing them.
 
         Return either tuple of WORD-sized, byte-swapped data (raw==False)
@@ -100,10 +113,7 @@ class WordsIO:
         if self.is_empty():
             raise EmptyError()
 
-        if size * WORD_SIZE > len(self):
-            raise LeftoverBytesError(
-                f"Requested {size} words, only {len(self)} words in object"
-            )
+        self.assert_data_is_available(size)
 
         if raw:
             return self.binary_data[: size * WORD_SIZE]
@@ -139,7 +149,12 @@ EVENTS_PARTS = {
     ),
     "energy": (
         "u32" * 3 + "f32",
-        ("Esum_trailing", "Esum_leading", "Esum_gap", "baseline",),
+        (
+            "Esum_trailing",
+            "Esum_leading",
+            "Esum_gap",
+            "baseline",
+        ),
     ),
     "Qsums": (
         "u32" * 8,
@@ -154,28 +169,38 @@ EVENTS_PARTS = {
             "QDCSum7",
         ),
     ),
-    "Ext Time": ("u32p16u16", ("Ext_TS_Lo", "Ext_TS_Hi",),),
+    "Ext Time": (
+        "u32p16u16",
+        (
+            "Ext_TS_Lo",
+            "Ext_TS_Hi",
+        ),
+    ),
 }
 
 # We now build all possible combinations out of these parts that can
 # show up in list mode data We also add a "0" option to just read the
 # first row, since this row contains the header length we need to read
 HEADER_FORMATS = {
-    0: (EVENTS_PARTS["first row"][0], EVENTS_PARTS["first row"][1]),
-    4: (EVENTS_PARTS["header"][0], EVENTS_PARTS["header"][1]),
-    6: (
+    0: bitstruct.CompiledFormatDict(
+        EVENTS_PARTS["first row"][0], EVENTS_PARTS["first row"][1]
+    ),
+    4: bitstruct.CompiledFormatDict(
+        EVENTS_PARTS["header"][0], EVENTS_PARTS["header"][1]
+    ),
+    6: bitstruct.CompiledFormatDict(
         EVENTS_PARTS["header"][0] + EVENTS_PARTS["Ext Time"][0],
         EVENTS_PARTS["header"][1] + EVENTS_PARTS["Ext Time"][1],
     ),
-    8: (
+    8: bitstruct.CompiledFormatDict(
         EVENTS_PARTS["header"][0] + EVENTS_PARTS["energy"][0],
         EVENTS_PARTS["header"][1] + EVENTS_PARTS["energy"][1],
     ),
-    12: (
+    12: bitstruct.CompiledFormatDict(
         EVENTS_PARTS["header"][0] + EVENTS_PARTS["Qsums"][0],
         EVENTS_PARTS["header"][1] + EVENTS_PARTS["Qsums"][1],
     ),
-    16: (
+    16: bitstruct.CompiledFormatDict(
         EVENTS_PARTS["header"][0]
         + EVENTS_PARTS["energy"][0]
         + EVENTS_PARTS["Qsums"][0],
@@ -183,7 +208,7 @@ HEADER_FORMATS = {
         + EVENTS_PARTS["energy"][1]
         + EVENTS_PARTS["Qsums"][1],
     ),
-    10: (
+    10: bitstruct.CompiledFormatDict(
         EVENTS_PARTS["header"][0]
         + EVENTS_PARTS["energy"][0]
         + EVENTS_PARTS["Ext Time"][0],
@@ -191,7 +216,7 @@ HEADER_FORMATS = {
         + EVENTS_PARTS["energy"][1]
         + EVENTS_PARTS["Ext Time"][1],
     ),
-    14: (
+    14: bitstruct.CompiledFormatDict(
         EVENTS_PARTS["header"][0]
         + EVENTS_PARTS["Qsums"][0]
         + EVENTS_PARTS["Ext Time"][0],
@@ -199,7 +224,7 @@ HEADER_FORMATS = {
         + EVENTS_PARTS["Qsums"][1]
         + EVENTS_PARTS["Ext Time"][1],
     ),
-    18: (
+    18: bitstruct.CompiledFormatDict(
         EVENTS_PARTS["header"][0]
         + EVENTS_PARTS["energy"][0]
         + EVENTS_PARTS["Qsums"][0]
@@ -212,11 +237,6 @@ HEADER_FORMATS = {
 }
 
 
-def unpack_header_word(word_data: Union[bytes, bytearray], word_number):
-    fmt, field_names = HEADER_FORMATS[word_number]
-    return bitstruct.unpack_dict(fmt, field_names, word_data)
-
-
 def calc_cfd_error(cfd_trigger_source_bits):
     return cfd_trigger_source_bits == 7
 
@@ -228,14 +248,6 @@ def calc_cfd_fraction(cfd_trigger_source_bits, cfd_fractional_time):
         return ((cfd_trigger_source_bits - 1) + cfd_fractional_time / 8192) * 2
 
 
-def calc_eventtime(evttime_lo, evvtime_hi):
-    return (evttime_lo + evvtime_hi * 2 ** 32) * 10
-
-
-def calc_external_timestamp(ext_ts_lo, ext_ts_hi):
-    return ext_ts_lo + ext_ts_hi * 2 ** 32
-
-
 class ListModeDataReader:
     """A binary data FIFO queue/transformer.
 
@@ -267,9 +279,19 @@ class ListModeDataReader:
         # try to get all the words for the next event
         # if the complete next event isn't in queue an exception is thrown but no data is discarded
         try:
-            event_header = unpack_header_word(self.words_stream.peek(1), 0)
+            event_header = HEADER_FORMATS[0].unpack(self.words_stream.peek(1))
             header_length = event_header["Header Length"]
             event_length = event_header["Event Length"]
+
+            # ensure we have enough data, this is needed, since we
+            # have two separate pop-calls (header, trace) and we need
+            # to make sure that both work. Otherwise we might get
+            # enough data for a header but not for the trace, in which
+            # case we would add more data an try again to read a
+            # header although trace data is next in the input queue
+            self.words_stream.assert_data_is_available(event_length)
+
+            # it's now save to read both header and trace
             header_words = self.words_stream.pop(header_length)
             trace_length = event_length - header_length
             if trace_length:
@@ -280,7 +302,26 @@ class ListModeDataReader:
             raise LeftoverBytesError("Partial event left at the end of queue")
 
         # unpack words/trace into their field values
-        event_values = unpack_header_word(header_words, header_length)
+        try:
+            event_values = HEADER_FORMATS[header_length].unpack(header_words)
+        except KeyError:
+            print("Something went wrong with the binary data.")
+            print("Debug information:")
+            print(f"   Current event header: {event_length}")
+            print(
+                f"   Number of bytes remaining in IO buffer:  {len(self.words_stream)}"
+            )
+            if len(self.words_stream):
+                filename = Path(".") / "pixie_binary_rest_debug.bin"
+                with filename.open("wb") as f:
+                    f.write(self.words_stream.binary_data)
+                print(f"   Saved IO buffer to:  {filename.absolute()}")
+            for i, data in enumerate(self.words_stream.last_data):
+                filename = Path(".") / f"pixie_binary_last_{i}_debug.bin"
+                with filename.open("wb") as f:
+                    f.write(data)
+                print(f"   Saved IO last elements to:  {filename.absolute()}")
+            raise
 
         if trace_data:
             trace = np.frombuffer(trace_data, f"({trace_length},2)<u2", count=1)[
@@ -296,13 +337,15 @@ class ListModeDataReader:
         CFD_fraction = calc_cfd_fraction(
             CFD_bits, event_values.pop("CFD Fractional Time")
         )
-        timestamp = calc_eventtime(
-            event_values.pop("EVTTIME_LO"), event_values.pop("EVTTIME_HI")
-        )
+        timestamp = (
+            event_values.pop("EVTTIME_LO") + event_values.pop("EVTTIME_HI") * 2**32
+        ) * 10
+
         if "Ext_TS_Lo" in event_values:
-            ext_timestamp = calc_external_timestamp(
-                event_values.pop("Ext_TS_Lo"), event_values.pop("Ext_TS_Hi")
+            ext_timestamp = (
+                event_values.pop("Ext_TS_Lo") + event_values.pop("Ext_TS_Hi") * 2**32
             )
+
             event_values.update({"ext_timestamp": ext_timestamp})
 
         for key in ["Event Length", "Header Length", "Trace Length"]:
@@ -326,6 +369,9 @@ class ListModeDataReader:
                 yield self.pop()
             except (EmptyError, LeftoverBytesError):
                 return
+            except TypeError as e:
+                print(f"TypeError: {e}", flush=True)
+                return
 
     def pop_all(self):
         return tuple(self.iterevents())
@@ -380,7 +426,7 @@ def events_from_files_generator(files, buffer_size=int(1e9)):
         reader.put(bytes_stream.pop(buffer_size))
         yield from reader.iterevents()
     if not reader.is_empty():
-        raise LeftoverBytesError()
+        print("[orange3]WARNING[/] Left over bytes in binary data stream.")
 
 
 def to_list(obj):
@@ -407,3 +453,32 @@ def read_list_mode_data_as_events(files, buffer_size=int(1e9), max_size=None):
     files = to_list(files)
     gen = events_from_files_generator(files, buffer_size)
     return list(islice(gen, max_size))
+
+
+def sort_events_by_channel(events, channel_list=None):
+    """Sort events by channel.
+
+    Takes the output of `read_list_mode_data_as_events` and sorts them
+    into a dictionary that has the channel number as key and as value
+    a list of events (still including the channel number).
+
+    Parameters
+    ----------
+    events
+        List of events as returned by `read_list_mode_data_as_events`
+    channel_list
+        Optional list of channels to include. This can help to reduce
+        the amount of data.
+
+    """
+
+    out = defaultdict(list)
+
+    for e in events:
+        ch = e.channel
+        if channel_list is not None and ch in channel_list:
+            out[ch].append(e)
+        else:
+            out[ch].append(e)
+
+    return out
